import org.apache.jena.query.Dataset;
import org.apache.jena.query.DatasetFactory;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryFactory;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.tdb.TDBFactory;

public class AnalyticsSSSP {

	public static void main(String[] args) {

		String tdb_folder = args[0];
		Dataset dataset = TDBFactory.createDataset(tdb_folder);

		// Remove graphs
		dataset = dataset.removeNamedModel("http://www.sparqual.org/graphs/global");

		Dataset datasetMem = DatasetFactory.createTxnMem();
		datasetMem = datasetMem.setDefaultModel(dataset.getDefaultModel());
		
		System.out.println("START");
		long timeStart = System.currentTimeMillis();

		String queryString = "CONSTRUCT {<http://www.sparqual.org/global> <http://www.sparqual.org/iter> ?depth} " + 
				"WHERE { VALUES (?depth) { (0) } }";
		Query query = QueryFactory.create(queryString);
		QueryExecution qe = QueryExecutionFactory.create(query, datasetMem);
		Model m = qe.execConstruct();

		datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/global", m);

		queryString = "CONSTRUCT {?node <http://www.sparqual.org/depth> ?depth . ?node <http://www.sparqual.org/path> \"\"} " + 
				"WHERE { VALUES (?node ?depth) { (<http://example.org/v6009541> 0)  } }";
		query = QueryFactory.create(queryString);
		qe = QueryExecutionFactory.create(query, datasetMem);
		m = qe.execConstruct();

		datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/visited", m);
		
		queryString = "CONSTRUCT {?node <http://www.sparqual.org/depth> ?depth . ?node <http://www.sparqual.org/path> \"\"} " + 
				"WHERE { VALUES (?node ?depth) { (<http://example.org/v6009541> 0)  } }";
		query = QueryFactory.create(queryString);
		qe = QueryExecutionFactory.create(query, datasetMem);
		m = qe.execConstruct();

		datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/visitnew", m);

		int count = 1;
		while(true) {

			System.out.println("Iter: " + count);

			// visit
			queryString = "CONSTRUCT { ?neighbour <http://www.sparqual.org/depth> ?depth2 . ?neighbour <http://www.sparqual.org/path> ?path2 } " +
					"WHERE { ?node ?p ?neighbour . " +
					" GRAPH <http://www.sparqual.org/graphs/global> { <http://www.sparqual.org/global> <http://www.sparqual.org/iter> ?depth } . " +
					" GRAPH <http://www.sparqual.org/graphs/visitnew> { ?node <http://www.sparqual.org/depth> ?depth . ?node <http://www.sparqual.org/path> ?path } . " +
					" BIND (?depth + 1 AS ?depth2) . BIND ( (CONCAT(?path,\" -[\",STR(?p),\"]-> \",STR(?neighbour))) AS ?path2 ) } ";
					

			query = QueryFactory.create(queryString);
			qe = QueryExecutionFactory.create(query, datasetMem);
			Model visit = qe.execConstruct();

			datasetMem = datasetMem.removeNamedModel("http://www.sparqual.org/graphs/visit");
			datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/visit", visit);

			// visitNew
			queryString = "CONSTRUCT { ?node <http://www.sparqual.org/depth> ?depth . ?node <http://www.sparqual.org/path> ?path } " +
					"WHERE " + 
					"  {  " + 
					"	{ GRAPH <http://www.sparqual.org/graphs/visit> { ?node <http://www.sparqual.org/depth> ?depth . ?node <http://www.sparqual.org/path> ?path } } MINUS " + 
					"	{ SELECT ?node WHERE { GRAPH <http://www.sparqual.org/graphs/visited> { ?node <http://www.sparqual.org/depth> ?depth2 } } } . " + 
					"  } ";

			query = QueryFactory.create(queryString);
			qe = QueryExecutionFactory.create(query, datasetMem);
			Model visitnew = qe.execConstruct();

			datasetMem = datasetMem.removeNamedModel("http://www.sparqual.org/graphs/visitnew");
			datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/visitnew", visitnew);

			// UNION of visitNew and visit
			queryString = "CONSTRUCT { ?s ?p ?o } " +
					"WHERE " + 
					"  {  " + 
					"	{ GRAPH <http://www.sparqual.org/graphs/visitnew> { ?s ?p ?o} } UNION " + 
					"	{ GRAPH <http://www.sparqual.org/graphs/visited> { ?s ?p ?o } } . " + 
					"  } ";

			query = QueryFactory.create(queryString);
			qe = QueryExecutionFactory.create(query, datasetMem);
			Model visited2 = qe.execConstruct();

			datasetMem = datasetMem.removeNamedModel("http://www.sparqual.org/graphs/visited");
			datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/visited", visited2);

			// depth value
			queryString = "CONSTRUCT { <http://www.sparqual.org/global> <http://www.sparqual.org/iter> ?depth2 } " +
					"WHERE " + 
					"  {  " + 
					"	GRAPH <http://www.sparqual.org/graphs/global> { <http://www.sparqual.org/global> <http://www.sparqual.org/iter> ?depth } " + 
					"	BIND(?depth + 1 AS ?depth2) " + 
					"  } ";

			query = QueryFactory.create(queryString);
			qe = QueryExecutionFactory.create(query, datasetMem);
			Model depthValue = qe.execConstruct();

			datasetMem = datasetMem.removeNamedModel("http://www.sparqual.org/graphs/global");
			datasetMem = datasetMem.addNamedModel("http://www.sparqual.org/graphs/global", depthValue);

			if(visitnew.size() == 0) {
				break;
			}
			count++;
		}

		queryString = "SELECT * " +
				"WHERE { GRAPH <http://www.sparqual.org/graphs/visited> { ?x ?y ?z } }";
		query = QueryFactory.create(queryString);
		qe = QueryExecutionFactory.create(query, datasetMem);

		ResultSet rs = qe.execSelect();
		// System.out.println(ResultSetFormatter.asText(rs));
		ResultSetFormatter.consume(rs);

		System.out.println(datasetMem.getNamedModel("http://www.sparqual.org/graphs/depth").size());
		
		System.out.println(System.currentTimeMillis() - timeStart);
		System.out.println("END");

		dataset.close();
		datasetMem.close();
		qe.close();


	}

}
